/*
 * Copyright (c) Citrix Systems, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */


#include "xen_internal.h"
#include "xen_vm_operations_internal.h"
#include <xen/api/xen_common.h>
#include <xen/api/xen_vm_operations_string_map.h>


xen_vm_operations_string_map *
xen_vm_operations_string_map_alloc(size_t size)
{
    xen_vm_operations_string_map *result = calloc(1, sizeof(xen_vm_operations_string_map) +
                                                  size * sizeof(struct xen_vm_operations_string_map_contents));
    result->size = size;
    return result;
}


void
xen_vm_operations_string_map_free(xen_vm_operations_string_map *map)
{
    if (map == NULL)
    {
        return;
    }

    size_t n = map->size;
    for (size_t i = 0; i < n; i++)
    {
        
        free(map->contents[i].val);
    }

    free(map);
}


static const struct_member vm_operations_string_map_struct_members[] =
    {
        { .type = &xen_vm_operations_abstract_type_,
          .offset = offsetof(xen_vm_operations_string_map_contents, key) },
        { .type = &abstract_type_string,
          .offset = offsetof(xen_vm_operations_string_map_contents, val) },
    };

const abstract_type vm_operations_string_map_abstract_type_ =
    {
       .typename = MAP,
       .struct_size = sizeof(vm_operations_string_map_struct_members),
       .member_count =
           sizeof(vm_operations_string_map_struct_members) / sizeof(struct_member),
       .members = vm_operations_string_map_struct_members
    };
